@@ -4,7 +4,7 @@ set -euxo pipefail
 
 rm -rf /tmp/rustc-pgo
 
-python2.7 ../x.py build --target=$PGO_HOST --host=$PGO_HOST \
+python3 ../x.py build --target=$PGO_HOST --host=$PGO_HOST \
     --stage 2 library/std --rust-profile-generate=/tmp/rustc-pgo
 
 RUSTC_BOOTSTRAP=1 ./build/$PGO_HOST/stage2/bin/rustc --edition=2018 \
@@ -12,17 +12,19 @@ RUSTC_BOOTSTRAP=1 ./build/$PGO_HOST/stage2/bin/rustc --edition=2018 \
 
 # Download and build a single-file stress test benchmark on perf.rust-lang.org.
 function pgo_perf_benchmark {
-    local PERF=e095f5021bf01cf3800f50b3a9f14a9683eb3e4e
+    local PERF=1e19fc4c6168d2f7596e512f42f358f245d8f09d
     local github_prefix=https://raw.githubusercontent.com/rust-lang/rustc-perf/$PERF
     local name=$1
+    local edition=$2
     curl -o /tmp/$name.rs $github_prefix/collector/benchmarks/$name/src/lib.rs
 
-    RUSTC_BOOTSTRAP=1 ./build/$PGO_HOST/stage2/bin/rustc --edition=2018 \
+    RUSTC_BOOTSTRAP=1 ./build/$PGO_HOST/stage2/bin/rustc --edition=$edition \
         --crate-type=lib /tmp/$name.rs
 }
 
-pgo_perf_benchmark externs
-pgo_perf_benchmark ctfe-stress-4
+pgo_perf_benchmark externs 2018
+pgo_perf_benchmark ctfe-stress-4 2018
+pgo_perf_benchmark inflate 2015
 
 cp -pri ../src/tools/cargo /tmp/cargo
 
